from fastapi import APIRouter, BackgroundTasks, Depends, Query
from sqlalchemy.orm import Session

import app.models.domain.devices as domain_devices
from app.api.dependencies.db import get_db
from app.controllers.equipment.ahu.supply_air_temperature_set import get_next_supply_air_temperature_set
from app.controllers.equipment.ahu.thermal_mode import get_thermal_mode
from app.controllers.equipment.fcu.early_start import get_recommended_early_start_time
from app.controllers.equipment.fcu.on_ratio import start_on_ratio_mode
from app.controllers.equipment.pau.freq_set import get_next_acatfu_freq_set
from app.controllers.equipment.pau.supply_air_temperature_set import get_next_acatfu_supply_air_temperature_set
from app.controllers.equipment.pau.switch import get_switch_action
from app.controllers.equipment.vrf.basic import get_vrf_instructions
from app.models.domain.devices import DevicesInstructionsBaseResponse, DevicesEarlyStartTime

router = APIRouter()


@router.get('/instructions/acatah/thermal-mode-set', response_model=DevicesInstructionsBaseResponse)
async def get_acatah_thermal_mode_set(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        device_id: str = Query(..., max_length=50, regex='^Eq', alias='equipId')
):
    thermal_mode = await get_thermal_mode(project_id, device_id)

    return {
        'projectId': project_id,
        'equipId': device_id,
        'output': {
            'thermalModeSet': thermal_mode
        }
    }


@router.get('/instructions/acatah/supply-air-temperature-set', response_model=DevicesInstructionsBaseResponse)
async def get_acatah_supply_air_temperature_set(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        device_id: str = Query(..., max_length=50, regex='^Eq', alias='equipId')
):
    next_supply_air_temperature_set = await get_next_supply_air_temperature_set(project_id, device_id)

    return {
        'projectId': project_id,
        'equipId': device_id,
        'output': {
            'supplyAirTemperatureSet': next_supply_air_temperature_set
        }
    }


@router.post('/on-ratio-experiment/fcu/{fcu_id}/target/{target}/period/{period}')
async def run_on_ratio_experiment(fcu_id: str, target: float, period: int, background_tasks: BackgroundTasks):
    background_tasks.add_task(start_on_ratio_mode, fcu_id, target, period)
    return {
        'fcu': fcu_id,
        'target': target,
        'period': period
    }


@router.get('/instructions/acatfu/equip-switch-set', response_model=DevicesInstructionsBaseResponse)
async def get_acatfu_equip_switch_set(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        device_id: str = Query(..., max_length=50, regex='^Eq', alias='equipId')
):
    action = await get_switch_action(project_id, device_id)

    return {
        'projectId': project_id,
        'equipId': device_id,
        'output': {
            'equipSwitchSet': action
        }
    }


@router.get('/instructions/acatfu/supply-air-temperature-set', response_model=DevicesInstructionsBaseResponse)
async def get_acatfu_supply_air_temperature_set(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        device_id: str = Query(..., max_length=50, regex='^Eq', alias='equipId')
):
    temperature_set = await get_next_acatfu_supply_air_temperature_set(project_id, device_id)

    return {
        'projectId': project_id,
        'equipId': device_id,
        'output': {
            'supplyAirTemperatureSet': temperature_set
        }
    }


@router.get('/instructions/acatfu/freq-set', response_model=DevicesInstructionsBaseResponse)
async def get_acatfu_freq_set(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        device_id: str = Query(..., max_length=50, regex='^Eq', alias='equipId')
):
    freq_set = await get_next_acatfu_freq_set(project_id, device_id)

    return {
        'projectId': project_id,
        'equipId': device_id,
        'output': {
            'fanFreqSet': freq_set
        }
    }


@router.get('/early-start/prediction/acatfc', response_model=DevicesEarlyStartTime)
async def get_acatfc_early_start_time(
        project_id: str = Query(..., max_length=50, regex='^Pj', alias='projectId'),
        space_id: str = Query(..., max_length=50, regex='^Sp', alias='spaceId'),
        db: Session = Depends(get_db)
):
    minutes = await get_recommended_early_start_time(db, project_id, space_id)
    return {
        'projectId': project_id,
        'spaceId': space_id,
        'minutes': minutes
    }


@router.post('/instructions/acatvi', response_model=domain_devices.ACATVIInstructionsTemporaryResponse)
async def get_acatvi_instructions(device_info: domain_devices.ACATVIInstructionsRequest):
    output = await get_vrf_instructions(
        device_info.space_temp_target,
        device_info.return_air_temp,
        device_info.space_realtime_temp
    )
    resp = {'output': output}

    return resp


@router.post('/instructions/acatfc', response_model=domain_devices.ACATFCInstructionsResponse)
async def get_acatfc_instructions(params: domain_devices.ACATFCInstructionsRequest):
    pass


@router.post('/instructions/acatva', response_model=domain_devices.ACATVAInstructionsResponse)
async def get_acatva_instructions(params: domain_devices.ACATVAInstructionsRequest):
    pass
